
#include <UTFT.h>
#include <ITDB02_Touch.h>

#include <avr/io.h>
#include <stdint.h> 
#include <stdlib.h>
//#include "timeout.h"   // comprend : #define F_CPU 16000000UL  // 16 MHz
#include <math.h>

// Declare which fonts we will be using
extern uint8_t SmallFont[];

// Arduino Mega:
// -------------------
// Standard Arduino Mega/Due shield            : <display model>,38,39,40,41
// CTE TFT LCD/SD Shield for Arduino Mega      : <display model>,38,39,40,41


const char *version = "1.1";  // ( signal entrant 10kHz à 90kHz )

UTFT TFT320(ITDB32S,38,39,40,41);
ITDB02_Touch TACTILE(6,5,4,3,2);

typedef float complexe[2];

complexe tab_X[128];  // nb_ech valeurs traitées
complexe tab_W[64];  // nb_ech/2


float frequence1; // Hz
float frq_echantillonnage; // Hz

const uint8_t nb_etapes=7; 
uint8_t nb_ech = pow (2,nb_etapes); // nombre d'échantillons = 2 puissance(nb_etapes)

uint8_t hamming;
uint8_t bourrage;
uint8_t mode_affi;

uint8_t etat_boutons; // 8 bits definissant chacun l'état d'un bouton, soit 8 boutons en tout

void init_ports (void)  // ports perso
// 0 = entree, 1=sortie   ;   les 1 sur les pins en entrees activent les R de Pull Up (tirage à VCC)
{
  PORTA = 0b00000111;
  DDRA  = 0b01111000;  //PA7 en entree (ADC7 - entrée signal); PA0, PA1, PA2 jumpers à GND.
  
  PORTB = 0b00000000;
  DDRB |= 0b11111111;
  
  PORTC = 0b00000000;
  DDRC =  0b11111111;

  PORTD = 0b00000000; 
  DDRD =  0b11111111;
}



/*
The ADC module contains a prescaler, which generates an acceptable ADC clock fre-
quency from any CPU frequency above 100 kHz. The prescaling is set by the ADPS bits
in ADCSRA
*/

void InitADC (void)
{
//  SFIOR  = 0b10000000; // ADTS2,1,0 = 100 -> CAN declenché par Timer0 Overflow - p:218
  ADMUX  = 0b01000111;   //Bit 7:6 = ADC Reference Selection = 01 -> AVCC ; //Bits 0:4 - Analog Channel Selection Bits et gain éventuel - p214
// ici Select pin ADC7 using MUX  avec ref tension interne = AREF externe - pas de gain.  

/** CHOISIR UNE SEULE LIGNE CI-DESSOUS pour ADCSRA */

// parametrage gamme 8-90 kHz
//  ADCSRA = 0b10000010;  // bit7: Activate ADC; ADPS2..0->Prescaler=f/4 ; bit5 = ADATE (ADC Auto Trigger Enable)->non utilisé ici - p:216 et 218

// parametrage gamme 4-56 kHz
//  ADCSRA = 0b10000011;  // bit7: Activate ADC; ADPS2..0->Prescaler=f/8 ; bit5 = ADATE (ADC Auto Trigger Enable)->non utilisé ici - p:216 et 218

// parametrage gamme 2-31 kHz
//  ADCSRA = 0b10000100;  // bit7: Activate ADC; ADPS2..0->Prescaler=f/16 ; bit5 = ADATE (ADC Auto Trigger Enable)->non utilisé ici - p:216 et 218
 
// parametrage gamme 1-15 kHz
//  ADCSRA = 0b10000101;  // bit7: Activate ADC; ADPS2..0->Prescaler=f/32 ; bit5 = ADATE (ADC Auto Trigger Enable)->non utilisé ici - p:216 et 218
 
// parametrage gamme 300 Hz-8000 Hz
//  ADCSRA = 0b10000110;  // bit7: Activate ADC; ADPS2..0->Prescaler=f/32 ; bit5 = ADATE (ADC Auto Trigger Enable)->non utilisé ici - p:216 et 218

// parametrage gamme 200 Hz- 4300 Hz
  ADCSRA = 0b10000111;  // bit7: Activate ADC; ADPS2..0->Prescaler=f/32 ; bit5 = ADATE (ADC Auto Trigger Enable)->non utilisé ici - p:216 et 218
 
}

void RAZ_tableau_echantillons()
{
   uint8_t n;
   for(n=0; n < nb_ech; n++)
   {
       tab_X[n][1] = 0; // partie reelle
       tab_X[n][2] = 0; // partie imaginaire
   }
}

uint8_t bit_reversal(uint8_t num, uint8_t nb_bits)
{
    uint8_t r = 0, i, s;
    if ( num > 1<< nb_bits) { return 0; }

    for (i=0; i<nb_bits; i++)
    {
        s = (num & (1 << i));
        if(s) { r |= (1 << ((nb_bits - 1) - i)); }
    }
    return r;
}



void acquisition()
{  
  uint16_t acqH;
  uint16_t tab_acq[64];
  uint8_t n, r, n_max;
  float x;
  uint16_t valeur_lue;  

  if (bourrage != 0) {n_max = nb_ech; } else {n_max = nb_ech /2; }
    //n_max  = 16;
    
    for(n=0; n < n_max-1; n++)
    {    
        ADCSRA |= _BV(ADSC);    //Start conversion - resolution 10bits (0..1024)
        while (ADCSRA & _BV(ADSC) ) {;}    // attend la fin de la converstion
        tab_acq[n] = ADCL;
        acqH = ADCH; // passage à 16 bits pour pouvoir décaller
        tab_acq[n] += acqH << 8;   // ici tab_acq[n] = 0..1024
    }    
    for(n=0; n < n_max-1; n++)
    {
      valeur_lue =tab_acq[n];
      x= (valeur_lue - 512.0) / 100.0;   // 512 = 1024/2
      if (hamming)  { x = x * (1- cos (2* M_PI * n / n_max ));}       
      r=bit_reversal(n,nb_etapes);
      tab_X[r][1] = x * 16; // partie reelle  - * 16 pour obtenir un nombre en virgule fixe
      tab_X[r][2] = 0; // partie imaginaire
    }
}



void remplir_tableau_echantillons()
{
// création du signal à analyser
    uint8_t n, r, n_max;
    float Te; // période d'échantillonage
    float y;
    float a;
    Te = 1.0 / frq_echantillonnage;

    if (bourrage != 0)   {	n_max = nb_ech /4; } 	else 	{	n_max = nb_ech; }

    for(n=0; n < n_max-1; n++) // nb d'échantillons
    {
        y=  cos( 2.0 * M_PI * frequence1 * n * Te ); 
        if (hamming)  { y = y * (1- cos (2* M_PI * n / n_max ));}
        

        r=bit_reversal(n,nb_etapes);
        y *= 2;
        
        tab_X[r][1] = y; // partie reelle
        tab_X[r][2] = 0; // partie imaginaire
     }
}



void calcul_tableau_W()
{
// calcul et memorisation dans un tableau des twiddle factors
    uint8_t n;
    //float x;
    float x, cosx, sinx;

    for(n=0; n<(nb_ech/2-1); n++)
    {
        x=2.0*M_PI * n / nb_ech;

        tab_W[n][1] =  cos(x);  // partie reelle
        tab_W[n][2] =  -sin(x);  // partie imaginaire
    }
}




void calcul_fourier()
{
    complexe produit; // voir le type "complexe" déclaré plus haut

    uint8_t etape, e1, e2, li, w, ci;
    uint8_t li_max=nb_ech;

    e2=1;

    for (etape=1; etape<=nb_etapes; etape++)
    {
        e1=e2; //(e1 evite d'effectuer des divisions e2/2 plus bas)
        e2=e2+e2;

        for (li=0; li<li_max; li+=1)
        {
            ci=li & (e2-1);  // ET bit à bit
            if (ci>(e1-1))
            {
                w=li_max/e2*(li & (e1 -1));  // ET bit à bit  calcul du numéro du facteur de tripatouillages W

                produit[1]=(tab_W[w][1]*tab_X[li][1]) - (tab_W[w][2]*tab_X[li][2]); 
                produit[2]=(tab_W[w][1]*tab_X[li][2]) + (tab_W[w][2]*tab_X[li][1]);

                tab_X[li][1]=tab_X[li-e1][1]-produit[1];
                tab_X[li][2]=tab_X[li-e1][2]-produit[2];

                tab_X[li-e1][1]=tab_X[li-e1][1]+produit[1];
                tab_X[li-e1][2]=tab_X[li-e1][2]+produit[2];
            }
        }
    }
}




void tracer_signal()
{

		uint8_t offset_y = 60;
    int32_t a, S;
    uint8_t  r,  memo_x, memo_y, n, x, y;

    x=0;
    y= offset_y;
    
    TFT320.setColor(200, 0, 0);
    TFT320.drawLine(1,offset_y,319,offset_y); 
    
    TFT320.setColor(255, 255, 0);
		TFT320.setBackColor(100, 100, 100);
    
    
    
    for(n=0; n<128; n++)
    {
        memo_x = x;
        memo_y = y;
        x = 2*n;
        r=bit_reversal(n, nb_etapes);
        if (hamming !=0) {a = 10.0;} else {a=20.0;}
        S = a * tab_X[r][1]; // partie réelle
        if ( (abs(S) )< 240 )  {y= lrint(S);} else {y=1;}
        
/** choisir une des lignes ci-dessous **/    

        //buffer_trace_baton_V(x, y);
       
       y= offset_y - y;
       TFT320.drawLine(memo_x, memo_y, x, y);

    }
  //  affi_buffer(84);
}




void tracer_tableau_valeurs()
{
	uint8_t offset_x = 100;
  uint8_t offset_y = 200;
  uint8_t n ;
  uint8_t memo_x, memo_y, x, y;
  float A;

  uint8_t z;

    if (bourrage != 0 ) {z=4;} else {z=1;}

    x=0;
    y= offset_y;
    
    TFT320.setColor(200, 0, 0);
    TFT320.drawLine(1,offset_y,319,offset_y);
    
    TFT320.setColor(0, 180, 200);
		TFT320.setBackColor(0, 0, 0);


    for(n=0; n<64; n++)  
    {
        memo_x = x;
        memo_y = y;
        
        //x = n;        
        x = offset_x + 2*n; 
    
        A = ((tab_X[n][1] * tab_X[n][1]) + (tab_X[n][2] * tab_X[n][2]));
        A = z * sqrt(A);
			 
        if ( (abs(A) )< 240 )  {y= lrint(A);} else {y=240;}
        
        y= offset_y - y;

/** choisir une des trois lignes ci-dessous; ATTENTION : les rapidités ne sont pas les mêmes !! **/    
      //  buffer_drawPixel(x, 47-y);    // la plus rapide, mais peu visible...
       // buffer_trace_baton_V(x, 1 + y);
       // buffer_trace_segment( memo_x, memo_y, x, y);
       
       //TFT320.drawPixel(x, 200-y); 
       TFT320.drawLine(memo_x, memo_y, x, y);
    }

	//	affi_buffer(84); // 12.6 ms
   
}


void trace_entete()
{
	TFT320.setColor(64, 64, 64);
  TFT320.fillRect(0, 0, 319, 13); // bandeau en haut
  
  TFT320.setColor(64, 64, 64);
  TFT320.fillRect(0, 226, 319, 239);  // bandeau en bas
  
  TFT320.setColor(255, 255, 255);
  TFT320.setBackColor(64, 64, 64);
  TFT320.print("Transformee de Fourier", CENTER, 1);
  
  TFT320.setColor(255,255,0);
  TFT320.print("ATmega2560 - Silicium628", CENTER, 227);
  	
}


void trace_reticule()
{
// cadre   
  TFT320.setColor(0, 150, 0);
  TFT320.setBackColor(0, 0, 0);
  TFT320.drawRect(0, 14, 319, 225);   
  
//reticule  
  TFT320.drawLine(159, 15, 159, 224);
  TFT320.drawLine(1, 119, 318, 119);
  
  for (int i=9; i<310; i+=10) {  TFT320.drawLine(i, 117, i, 121); }
  for (int i=19; i<220; i+=10) {  TFT320.drawLine(157, i, 161, i); }
}


void trace_bouton(uint8_t n, uint8_t etat)
{
	uint16_t position_x = 290;
	uint8_t dy = 25 * n;
  if (etat != 0) { TFT320.setColor(0, 255, 0); } else { TFT320.setColor(0, 80, 0); }
  TFT320.fillRoundRect(position_x, 20+dy, position_x+20, 40+dy);
  TFT320.setColor(255, 255, 255);
  TFT320.drawRect(position_x, 20+dy, position_x+20, 40+dy);
 
}



void trace_ecran()
{
	TFT320.setColor(0, 0, 0);
	TFT320.setBackColor(0, 0, 0);
	TFT320.fillRect(0, 14, 319, 225);
	
	trace_entete();
  trace_reticule();
  
	trace_bouton(0, hamming);
	trace_bouton(1, bourrage);
	
	TFT320.setColor(255, 255, 255);
  TFT320.print("H", 265, 25);
  TFT320.print("B", 265, 50);
	
}



void setup()
{
  randomSeed(analogRead(0));
  
  InitADC();
  
  TFT320.InitLCD();
  TFT320.setFont(SmallFont);
  TFT320.clrScr();
  
  TACTILE.InitTouch(LANDSCAPE);
  TACTILE.setPrecision(PREC_MEDIUM);
  
  trace_ecran();
}


void polling_boutons()
{
	uint8_t xt0 = 8; // position horizontale des boutons (alignés verticalement)
	uint8_t yt0;
	uint8_t ds = 8;	// espacement vertical des boutons
	uint8_t xt, yt;
	
	while (TACTILE. dataAvailable() == true)
	{
		TACTILE.read();
		xt = TACTILE.getX();
		yt = TACTILE.getY();
		
		TFT320.printNumI(xt, 280, 215, 3, ' ');
		TFT320.printNumI(yt, 280, 225, 3, ' ');

		yt0 = 28; // position verticale (centre) du premier bouton
		if ((xt > xt0-ds) && (xt < xt0+ds) && (yt > yt0-ds) && (yt < yt0+ds))
		{ 
			etat_boutons ^=0b00000001;
			hamming = etat_boutons & 0b00000001;
			trace_ecran();
			delay(300);
		}	

		yt0 = 54; // position verticale (centre) du second bouton
		if ((xt > xt0-ds) && (xt < xt0+ds) && (yt > yt0-ds) && (yt < yt0+ds))
		{ 
			etat_boutons ^=0b00000010;
			bourrage = etat_boutons & 0b00000010;
			trace_ecran();
			delay(300);
		}	
	}
	delay(100);	

}


void loop()
{
  
  randomSeed(analogRead(0));

  frequence1 = 500.0; // Hz
  frq_echantillonnage=3200.0; // Hz
  
  calcul_tableau_W();
   
  //frequence1 = 310.0; // Hz
            
 //     for(uint8_t n=1; n<23; n++)
 //    {

        //frequence1 += 30;
 
      
		RAZ_tableau_echantillons();
//	acquisition();  // 7ms
		remplir_tableau_echantillons();      
		tracer_signal(); // entrée ATTENTION : il faut faire cet affichage AVANT de calculer la TF parce que les résultats écrasent le tableau d'entrée
		calcul_fourier();  // 7 ms avec calculs en virgule fixe (int 16 bits) au lieu de 100..120ms avec calculs en virgule flotante (float 32 bits)
		tracer_tableau_valeurs(); // 16ms 

    polling_boutons();

}

